/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: DacConnection.java,v 1.3 2005/09/17 17:35:49 rampil Exp $
 */
package org.logical_paradox.vsan.dac;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import org.logical_paradox.common.io.ExpandableBuffer;
import org.logical_paradox.common.util.ByteStreamConverter;
import org.logical_paradox.vsan.dac.command.Protocol;
import org.logical_paradox.vsan.domain.exception.DomainAccessException;

/**
 * DACɐڑ邽߂̐ڑIuWFNg
 * Connectionɑ΂鑀DAC̃vgRɕϊĒʐMs
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.3 $
 */
public class DacConnection {
	/** DACɐڑĂ\Pbg */
	private Socket socket;
	/** ڑ */
	private String host;

	/**
	 * RXgN^
	 * @param host ڑm[h̃AhX
	 * @throws IOException ڑɎs
	 */
	protected DacConnection(String host) throws IOException {
		if(host == null || host.length() == 0) {
			throw new IllegalArgumentException();
		}

		String[] tokens = host.split(":");
		if(tokens == null || tokens.length < 2) {
			socket = new Socket(host, DacConfig.DEFAULT_PORT_NUMBER);
		} else {
			socket = new Socket(tokens[0], Integer.parseInt(tokens[1]));
		}

		this.host = host;
	}
	/**
	 * DACɐڑ
	 * @param host ڑm[h̃AhX
	 * @return ڑIuWFNg
	 * @throws IOException ڑɎs
	 */
	public static final DacConnection open(String host) throws IOException {
		return new DacConnection(host);
	}
	/**
	 * r
	 * ŕAڑ敶ł̔rƂ
	 * @param o rΏ
	 * @return true: / false:Ⴄ
	 */
	public boolean equals(Object o) {
		DacConnection con = (DacConnection)o;
		return getHost().equals(con.getHost());
		
	}
	/**
	 * nbVR[hԂD
	 * @return nbVR[h
	 */
	public int hashCode() {
		return getHost().hashCode();
	}
	/**
	 * ڑm[h̃AhXԂ
	 * @return ڑm[h̃AhX
	 */
	public String getHost() {
		return host;
	}
	/**
	 * œKXgAR[hԍĕԂD
	 * w肳ꂽTCY̋󂫗̈D
	 * @return IꂽR[hԍ(-1: œKXgAR[hԍȂ)
	 */
	public long getBestStoreRecordNo(int size) {
		return -1;
	}
	/**
	 * ʒuf[^
	 * @param recordBegin ݊Jnʒu
	 * @param bstream ރf[^
	 * @throws DomainAccessException ݂Ɏs
	 */
	public void write(long recordBegin, byte[] bstream) throws DomainAccessException {
		/*
		 * Mf[^̍쐬
		 */
		ByteBuffer bb = ByteBuffer.allocate(1 + 8 + 4 + bstream.length).order(ByteOrder.BIG_ENDIAN);
		bb.put((byte)Protocol.P_STORE);
		bb.putLong(recordBegin);
		bb.putInt(bstream.length);
		bb.put(bstream);

		// DACɃR}h𑗐M
		Response response = null;
		try {
			response = send(bb.array(), true);
		} catch(IOException e) {
			throw new DomainAccessException(e.getMessage());
		}

		if(response.getResponseCode() == ResponseCode.OK) {
			return;
		} else {
			throw new DomainAccessException("fatal error is caused in executing STORE command: code=[" + response.getResponseCode() + "]");
		}
	}
	/**
	 * ʒuf[^ǂݍ
	 * @param recordBegin ǂݍ݊Jnʒu
	 * @param bstream ǂݍ񂾃f[^i[obt@
	 * @return ǂݍ񂾃f[^
	 */
	public int read(long recordBegin, int records, byte[] bstream) throws DomainAccessException {
		/*
		 * Mf[^̍쐬
		 */
		ByteBuffer bb = ByteBuffer.allocate(1 + 4 + (records * 8)).order(ByteOrder.BIG_ENDIAN);
		bb.put((byte)Protocol.P_RESTORE);
		bb.putInt(records);

		// R[hXg
		for(int i = 0; i < records; i++) {
			bb.putLong(recordBegin + i);
		}

		// DACɃR}h𑗐M
		Response response = null;

		try {
			response = send(bb.array(), true);
		} catch(IOException e) {
			throw new DomainAccessException(e.getMessage());
		}

		if(response.getResponseCode() != ResponseCode.OK) {
			throw new DomainAccessException("fatal error is caused in executing RESTORE command: code=[" + response.getResponseCode() + "]");
		}

		ByteBuffer buffer = ByteBuffer.allocate(response.getData().length).order(ByteOrder.BIG_ENDIAN);
		buffer.put(response.getData());
		buffer.rewind();

		// R[h
		int replyRecords = buffer.getInt();
		int offset = 0;
		boolean exitf = false;

		for(int i = 0; i < replyRecords && exitf == false; i++) {
			// R[hǂ
			// ubNTCY / R[hԍ / f[^TCY
			buffer.getInt();
			buffer.getLong();
			int datasize = buffer.getInt();
			byte[] dumpTarget = new byte[datasize];

			buffer.get(dumpTarget);
			int length = 0;
			if(offset + dumpTarget.length >= bstream.length) {
				exitf = true;
				length = bstream.length - offset;
			} else {
				length = dumpTarget.length;
			}

			System.arraycopy(dumpTarget, 0, bstream, offset, length);

			offset += length;
		}

		return offset;
	}
	/**
	 * DACɃR}h𑗐MD
	 * @param data f[^(R}hR[h܂)
	 * @param ignore true:I/OG[𖳎 / false:ȂŗO𑗏o
	 * @return DAC̉
	 * @throws IOException ʐMQ
	 */
	protected Response send(byte[] data, boolean ignore) throws IOException {
		byte[] rc = new byte[0];
		int resultcode = -1;

		try {
			// ʐM
			BufferedOutputStream bw = new BufferedOutputStream(socket.getOutputStream());
			bw.write(data);
			bw.flush();

			// f[^M
			ExpandableBuffer exb = new ExpandableBuffer();

			// pPbgM
			BufferedInputStream bi = new BufferedInputStream(socket.getInputStream());

			// sXe[^XM
			byte[] status = new byte[4];
			bi.read(status);
			resultcode = ByteStreamConverter.getInt(status);

			byte[] packetnum = new byte[4];
			bi.read(packetnum);

			int packets = ByteStreamConverter.getInt(packetnum);

			// pPbg̓ǂݍ
			byte[] packetsz = new byte[4];
			for(int i = 0; i < packets; i++) {
				bi.read(packetsz);
				int packetlen = ByteStreamConverter.getInt(packetsz);
				byte[] body = new byte[packetlen];

				// f[^̎M
				bi.read(body);
				exb.write(body);
			}

			rc = exb.byteStream();
		} catch(IOException e) {
			if(ignore == false) {
				throw e;
			}
		}
		return new Response(resultcode, rc);
	}
	/**
	 * DACƂ̐ڑ
	 * DACɂڑ̖肪̂ŁAKN[Y邱
	 * @throws IOException N[YɎs
	 */
	public void close() throws IOException {
		if(socket != null && socket.isClosed() == false) {
			// DAC̃ZbV
			send(new byte[]{Protocol.P_CLOSE_SESSION}, false);

			socket.close();
		}
	}
}
