/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: DiskArrayController.java,v 1.3 2005/09/28 16:14:29 rampil Exp $
 */
package org.logical_paradox.vsan.dac;

import java.io.File;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.rmi.RemoteException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.logical_paradox.common.session.SessionManager;
import org.logical_paradox.common.util.Cache;
import org.logical_paradox.common.util.StringUtils;
import org.logical_paradox.vsan.domain.DomainController;
import org.logical_paradox.vsan.node.Database;
import org.logical_paradox.vsan.node.DatabaseAccessException;


/**
 * fBXNACRg[[
 * ̃vgRMāAf[^t@C̑{T[o[<br>
 * hCRg[[Ƃ̒ʐM{
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.3 $
 */
public class DiskArrayController {
	/** fBXNACRg[[̏ -  */
	public static final int E_INIT = 0;
	/** fBXNACRg[[̏ - NCAgڑ҂ */
	public static final int E_WAITING = 1;
	/** fBXNACRg[[̏ - I */
	public static final int E_CLOSING = 100;
	/** fBXNACRg[[̏ - ~ */
	public static final int E_CLOSED = 255;

	/** ftHg̃NGXg󂯎҂(1) */
	public static final int DEFAULT_CONNECTION_WAIT_MSEC = 60000;
	/** T[o[\Pbgaccept҂Ԍp */
	public static final int SERVER_SOCKET_TIMEOUT_SEC = 5000;

	/** ݂̃T[o[Xe[^X */
	private int serverStatus = E_INIT;

	/** T[o[\Pbg */
	private ServerSocket serverSocket = null;

	/** K[ */
	private static final Log log = LogFactory.getLog(DiskArrayController.class);

	/** RtBO[V */
	private final DacConfig config;
	/** m[hReLXg */
	private final NodeContext context;

	/** ҋ@̃XbhXg */
	private Cache waitingThreads = new Cache();
	/** Xi[ */
	private Listener[] listeners;
	/** NGXgvZbT̋󂫑҂(msec) */
	private int waittime = DEFAULT_CONNECTION_WAIT_MSEC;
	/** T[o[\Pbgacceptf^C~O */
	private int serverSocketTimeoutSec = SERVER_SOCKET_TIMEOUT_SEC;

	/** hCRg[[ */
	private DomainController domainController = null;

	/** ZbV}l[W[ */
	private static final SessionManager sessionManager = new SessionManager();

	/**
	 * RXgN^
	 * @param config_ RtBO[V
	 * @throws Exception AvP[VŏłȂO
	 */
	public DiskArrayController(DacConfig config_) throws DatabaseAccessException {
		config = config_;
		context = new NodeContext(config);
	}
	/**
	 * fBXNACRg[[D
	 * @throws DatabaseAccessException f[^x[X̓ǂݍ݂Ɏs
	 */
	protected void init() throws DatabaseAccessException {
		// m[hReLXgɑ΂ăf[^x[Xݒ肷
		context.setDatabase(loadDatabase(context));
		String domain = context.getDatabase().getDomain();

		if(config.isJoinOnStartup() == true && StringUtils.isEmpty(domain) == false) {
			// hCɎڑ
			log.info("hC[" + domain + "]Ɏڑ܂");
			log.info("hCRg[[ = [" + config.getDomainController() + "]");

			try {
				DomainSynchronizer.sync(context);
				context.join(true);
			} catch (Exception e) {
				log.warn("hCRg[[ƒʐMs\Ȃ߁ChCɎQł܂ł");
				log.warn("hCRg[[NC蓮ŃhC֎QĂ", e);

				domainController = null;
			}
		}
	}
	/**
	 * ZbV}l[W[Ԃ
	 * @return ZbV}l[W[
	 */
	public static final SessionManager getSessionManager() {
		return sessionManager;
	}
	/**
	 * hCf[^x[Xt@C[h
	 * ̃fBNgɉ݂ȂꍇÃf[^x[XԂ
	 * @param context m[hReLXg
	 * @return [h܂͏ꂽf[^x[XCX^X
	 */
	protected Database loadDatabase(NodeContext context) throws DatabaseAccessException {
		Database db = new Database(context);

		// ̃fBNgɑ݂f[^t@CT
		File dbdir = new File(context.getConfig().getDatabaseDir());
		File[] files = dbdir.listFiles();

		for(int i = 0; i < files.length; i++) {
			File f = files[i];
			String filename = f.getName();
			if(f.isFile() == true && f.getName().endsWith(DacConstant.DBI_EXT) == true) {
				String tokens[] = filename.split("\\.");
				db.setDomain(tokens[tokens.length -2]);
				break;
			}
		}
		if(StringUtils.isEmpty(db.getDomain()) == false) {
			try {
				log.info("̃hCf[^x[Xt@C[" + db.getFilename() + "]𕜌܂");
				db.restore();
				log.info("hCf[^x[Xt@C̕");
			} catch (IOException e) {
				log.warn("hCf[^x[Xt@C[" + db.getFilename() + "]̕Ɏs:", e);
			}
		}
		return db;
	}
	/**
	 * ̃fBXNACRg[[̃RtBO[VԂ
	 * @return ݂̃RtBO[V
	 */
	protected DacConfig getConfig() {
		return config;
	}
	/**
	 * Rg[[n
	 * @throws Exception AvP[VŏłȂO
	 */
	public void start() throws Exception {
		log.info("fBXNACRg[[Ă܂");

		init();

		// Ԃ֑Jڂ
		setServerStatus(E_INIT);

		// KṽXi[쐬ăv[OĂ
		// 쐬Xi[͂ɑҋ@Xg֓
		listeners = new Listener[config.getConnections()];
		for(int i = 0; i < config.getConnections(); i++) {
			listeners[i] = new Listener(this, context, i+1);
			waitingThreads.add(listeners[i]);
		}
		// Xi[Jn
		for(int i = 0; i < listeners.length; i++) {
			listeners[i].start();
		}

		log.info("Xi[[" + listeners.length + "]܂");
		waiting();

		// I
		shutdown();
	}
	/**
	 * XjOԂɓ
	 * ~MM܂ł̊ԁCڑƃf[^M҂
	 */
	protected void waiting() {
		// ڑ҂Ԃ֑Jڂ
		setServerStatus(E_WAITING);
		log.info("fBXNACRg[[̓Cxgҋ@ł");

		try {
			serverSocket = new ServerSocket(config.getPort());
			serverSocket.setSoTimeout(serverSocketTimeoutSec);
		} catch (IOException e) {
			log.error("T[o[\Pbg̐Ɏs: ", e);
			return;
		}
		Socket clientSocket = null;
		while(getServerStatus() == E_WAITING) {
			try {
				clientSocket = serverSocket.accept();
				log.info("Vڑ󂯕t܂ [" + clientSocket.toString() + "]");
				Listener listener = getListener();

				// Jn
				listener.accept(clientSocket);
			} catch(SocketTimeoutException e) {
				// T[o[\Pbg̒_
				// ̂܂܌p
				continue;
			} catch (IOException e) {
				log.warn("ڑɉ炩̓o̓G[܂", e);
			} catch (InterruptedException e) {
				// T[o[busŷ߁CڑtȂ
				log.warn("ڑāAXi[̊蓖Ăł܂ł");
				try {
					clientSocket.close();
				} catch (IOException e1) {
					// ؒf̂ߖ
				}
			}
		}
		// T[o[\Pbg
		try {
			serverSocket.close();
		} catch(IOException e) {
			log.warn("T[o[\Pbg̃N[Yɉ炩I/OG[܂", e);
		}
	}
	/**
	 * fBXNACRg[[Vbg_E
	 */
	protected void shutdown() {
		setServerStatus(E_CLOSING);
		log.info("fBXNACRg[[Vbg_EĂ܂");

		// SẴXi[N[Y
		for(int i = 0; i < listeners.length; i++) {
			listeners[i].tearDown();
		}
		// Xi[̏I҂
		for(int i = 0; i < listeners.length; i++) {
			try {
				listeners[i].join();
			} catch (InterruptedException e) {
				// 
			}
		}
		log.info("SẴXi[~܂");

		// hCf[^x[Xt@C
		String domain = context.getDatabase().getDomain();

		log.info("hCf[^x[X~Ă܂");
		context.getDatabase().close();
		log.info("hCf[^x[X~܂");

		if(domainController != null && StringUtils.isEmpty(domain) == false) {
			// hC痣E
			log.info("hC[" + domain + "]痣E܂");
			try {
				domainController.leave(domain, config.getHost() + ":" + config.getPort());
			} catch (RemoteException e) {
				log.warn("hC̗EɎs܂");
				log.warn("Kvł΁A蓮ŗEĂ");
			}

			domainController = null;
		}

		// SẴXi[ɒ~ꍇAXe[^XύX
		setServerStatus(E_CLOSED);
	}
	/**
	 * T[o[Xe[^XԂ
	 * @return T[o[Xe[^X
	 */
	public int getServerStatus() {
		return serverStatus;
	}
	/**
	 * T[o[Xe[^Xݒ肷
	 * @param status T[o[Xe[^X
	 */
	public void setServerStatus(int status) {
		serverStatus = status;
	}
	/**
	 * ҋ@XbhXgԂ
	 * @return ҋ@XbhXg
	 */
	Cache getWaitingThreads() {
		return waitingThreads;
	}
	/**
	 * ҂󂯃Xgɒǉ
	 * @param listener Xi[
	 */
	void addWaitingThread(Listener listener) {
		log.info("Xi[" + listener.getListenerNo() + "ҋ@Ԃɓ܂");
		waitingThreads.add(listener);
	}
	/**
	 * ҋ@ɂȂĂ郊Xi[擾
	 * 󂢂Ă郊Xi[ȂꍇA莞ԑҋ@āAł擾łȂꍇ
	 * ڑBUSYƂ
	 * @return 擾łXi[
	 * @throws InterruptedException 񂴂҂󂫂Ȃ
	 */
	protected Listener getListener() throws InterruptedException {
		synchronized(waitingThreads) {
			if(waitingThreads.size() == 0) {
				waitingThreads.wait(waittime);
			}

			return (Listener)waitingThreads.getOne();
		}
	}
	/**
	 * C\bh
	 * T[o[N邽߂̃u[gXgbv
	 * @param args vO
	 * @throws Exception ȂO
	 */
	public static final void main(String[] args) throws Exception {
		new DiskArrayController(new DacConfig(args)).start();
		log.info("fBXNACRg[[͒~܂");
	}
}
