/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: DomainConnectionProxyImpl.java,v 1.2 2005/09/07 13:38:06 rampil Exp $
 */
package org.logical_paradox.vsan.dfs;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import org.logical_paradox.vsan.dfs.exception.NoSuchAreaException;
import org.logical_paradox.vsan.domain.Domain;

/**
 * hCڑIuWFNg̃vLV̎
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.2 $
 */
public class DomainConnectionProxyImpl extends UnicastRemoteObject implements DomainConnectionProxy {
	/** ftHg̐ڑ^CAEgb(msec) 1 */
	public static final long DEFAULT_CONNECTION_TIMEOUT_SEC = 1000 * 60 * 60;

	/** hCڑIuWFNg */
	private final DomainConnection con;
	/** NCAgm[hID */
	private String clientId;
	/** ŏIʐM */
	private long lastUpdate = 0L;
	/** ڑ^CAEg҂b(msec */
	private long connectionTimeOutSec = DEFAULT_CONNECTION_TIMEOUT_SEC;
	/** \[XΏۃtO(true:GCŉ / false:^CAEg܂ŉȂ) */
	private boolean garbageTarget = false;
	/** hCt@CVXeRtBO[V */
	private final DfsConfig config;

	/**
	 * RXgN^D
	 * @param con_ hCڑIuWFNg
	 * @param cni NCAgID
	 * @param config_ RtBO[V
	 * @throws RemoteException CX^XɎs
	 */
	public DomainConnectionProxyImpl(DomainConnection con_, String cni, DfsConfig config_) throws RemoteException {
		con = con_;
		clientId = cni;
		config = config_;

		connectionTimeOutSec = config.getDomainSessionTimeout();

		touch();
	}
	/**
	 * ʐM}[Nt^D
	 */
	public void touch() {
		lastUpdate = System.currentTimeMillis();
	}
	/**
	 * ֘AÂĂhCڑIuWFNgC̃NCAgD
	 */
	public void unshare() {
		con.unshare(clientId);
	}
	/**
	 * ̐ڑIuWFNgL؂ꂩǂԂD
	 * L؂ĂꍇCZbV^CAEgƂCGC̑ΏۂƂȂD
	 * @return true:L؂ / false:LB
	 */
	public boolean isExpired() {
		return lastUpdate + connectionTimeOutSec < System.currentTimeMillis();
	}

	/**
	 * ̃vLVjD
	 * hCڑIuWFNg̋LÃvLVgpĂNCAgm[hD<br>
	 * ̃\bhĂяoƁAv}[ÑIuWFNgɐݒ肳B<br>
	 * ڑǗXbhv}[Nݒς݂̃vLV𔭌ۂɁA߂ăvLVjB
	 * @throws RemoteException ڑɎs
	 */
	public void disconnect() {
		garbageTarget = true;
	}
	/**
	 * \[XΏۂƂȂĂ邩ǂԂD
	 * \[XΏۂƂȂĂꍇCGCŒɂ̃IuWFNg͉D
	 * @return true:GCŉ / false:^CAEg܂ŉȂ
	 */
	public boolean isGarbageTarget() {
		return garbageTarget;
	}
	/**
	 * ̃vLVgpĂNCAgm[hIDԂD
	 * @return NCAgID
	 */
	public String getClientId() {
		return clientId;
	}
	/**
	 * ̃vLVɊ֘AÂĂChCڑIuWFNgԂD
	 * @return hCڑIuWFNg
	 */
	public DomainConnection getConnection() {
		touch();
		return con;
	}
	/**
	 * hCɊ蓖ĂĂt@CTCYԂD
	 * @return t@CTCY
	 * @throws RemoteException t@CTCY擾ɎsꍇɔO
	 */
	public long size() throws RemoteException {
		touch();
		return con.size();
	}
	/**
	 * oCgf[^ǂݍށD
	 * őŔzŎw肵TCYoCgǂݍŊi[D
	 * @param seekp V[N|C^
	 * @param length ǂݍރf[^̃TCY(P:oCg)
	 * @return 擾łhCf[^Xg[
	 * @throws RemoteException ǂݍ݂Ɏs
	 */
	public DomainDataStream read(long seekp, int length) throws RemoteException {
		int recordSize = con.getDomain().getRecordSize();
		long recordBegin = calcRecordNum(seekp);
		int beginOffset = 0;
		
		if(config.isFixed() == false) {
			// ϒQƃ[h̏ꍇAItZbgl
			beginOffset = (int)(seekp % recordSize);
		}

		// ǂݍ݂ɕKvȃobt@TCYvZ
		int records = length / recordSize;
		if(length % recordSize > 0) {
			records++;
		}
		int bytes = records * recordSize;
		byte[] bb = new byte[bytes];

		// DomainConnectionɑ΂āAf[^ǂݍ݂v
		// ԋpl(readcnt)́Af[^TCY
		int readcnt = con.read(recordBegin, bb);

		// ԋpꂽf[^w肳ꂽItZbĝق傫ꍇA
		// Yf[^݂ȂƂɂȂ̂EOF
		if(readcnt - beginOffset < 0) {
			throw new NoSuchAreaException();
		}

		byte[] b = new byte[length];

		// ԋpꂽf[^AKvȕԂ
		int realLength = (readcnt - beginOffset < length) ? readcnt - beginOffset : length;
		System.arraycopy(bb, beginOffset, b, 0, realLength);

		// ʃZbg쐬
		DomainDataStream dstream = new DomainDataStream(realLength, b);
		return dstream;
	}
	/**
	 * w背R[hoCgf[^ǂݍށD
	 * @param recordBegin JnR[hԍ
	 * @param length ǂݍރf[^̃TCY(P:oCg)
	 * @return 擾łhCf[^Xg[
	 * @throws RemoteException ǂݍ݂Ɏs
	 */
	public DomainDataStream readRecords(long recordBegin, int length) throws RemoteException {
		// V[N|C^vZ
		long seekp = con.getDomain().getRecordSize() * recordBegin;
		return read(seekp, length);
	}
	/**
	 * oCgށD
	 * @param seekp V[N|C^(-1:󂢂Ăʒu֓Kɏ)
	 * @param b oCg
	 * @return f[^݂̏JnꂽR[hԍ
	 * @throws RemoteException ݂Ɏs
	 */
	public long write(long seekp, byte[] b) throws RemoteException {
		int recordSize = con.getDomain().getRecordSize();
		if(seekp < 0) {
			// 󂢂ĂʒuIɌ
			seekp = con.computeBestSeekPointer(b.length);
		}

		long recordBegin = calcRecordNum(seekp);

		if(config.isFixed() == false) {
			// ϒQƃ[h̏ꍇCItZbgvZĂāC̃R[hƃf[^}[WKv
			int beginOffset = (int)(seekp % recordSize);

			// TODO ϒQƃ[h͖
		}

		// DACƒʐMāAf[^zXg[Wɏ
		con.write(recordBegin, b);

		return seekp;
	}
	/**
	 * oCgށD
	 * @param recordBegin 
	 * @param b oCg
	 * @throws RemoteException ݂Ɏs
	 */
	public long writeRecords(long recordBegin, byte[] b) throws RemoteException {
		// V[N|C^vZ
		long seekp = con.getDomain().getRecordSize() * recordBegin;
		return write(seekp, b);
	}
	/**
	 * V[N|C^烌R[h̊JnʒuvZĕԂD
	 * @param seekp V[N|C^
	 * @return R[hԍ
	 */
	protected long calcRecordNum(long seekp) {
		Domain domain = con.getDomain();
		long recordBegin = seekp / domain.getRecordSize();

		if(config.isFixed() == true) {
			// Œ蒷Qƃ[h̏ꍇAR[hvZŒ[1i߂
			recordBegin += seekp % domain.getRecordSize() > 0 ? 1 : 0;
		}
		return 1 + recordBegin;
	}
}
