/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: DistributedRandomAccessFile.java,v 1.3 2005/09/17 17:35:49 rampil Exp $
 */
package org.logical_paradox.vsan.vfi;

import java.io.EOFException;
import java.io.IOException;
import java.rmi.RemoteException;

import org.logical_paradox.common.io.RandomAccessFileDelegator;
import org.logical_paradox.common.util.ByteStreamConverter;
import org.logical_paradox.vsan.dfs.DomainConnectionProxy;
import org.logical_paradox.vsan.dfs.DomainDataStream;
import org.logical_paradox.vsan.dfs.DomainFileSystem;
import org.logical_paradox.vsan.dfs.exception.NoSuchAreaException;
import org.logical_paradox.vsan.lookup.LookupFactory;

/**
 * U_ANZXt@C
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.3 $
 */
public class DistributedRandomAccessFile implements RandomAccessFileDelegator {
	/** ڑhCt@CVXe */
	private final String dfsuri;
	/** NCAgID */
	private final String clientId;
	/** hCt@CVXe */
	private DomainFileSystem dfs;
	/** ڑvLV */
	private DomainConnectionProxy proxy;

	/** ݂̃V[N|C^ */
	private long seekp = 0;

	/**
	 * RXgN^
	 * @param dfsuri_ ڑhCt@CVXe
	 * @param clientId_ NCAgID
	 */
	public DistributedRandomAccessFile(String dfsuri_, String clientId_) throws RemoteException {
		dfsuri = dfsuri_;
		clientId = clientId_;

		// hCڑ
		open();
	}
	/**
	 * hCt@CVXeɐڑD
	 * @throws RemoteException ڑɎs
	 */
	protected void open() throws RemoteException {
		dfs = LookupFactory.getLookup().lookupDomainFileSystem(dfsuri);
		proxy = dfs.getConnection(clientId);
	}
	/**
	 * hCt@CVXeƂ̐ڑN[Y
	 * @throws IOException N[YɎs
	 */
	public void close() throws IOException {
		proxy.disconnect();
	}

	/**
	 * hCŜ̐LTCYԂD
	 * @return TCY
	 * @throws IOException TCY̎擾Ɏs
	 */
	public long length() throws IOException {
		return proxy.size();
	}

	/**
	 * V[N|C^ړD
	 * @return V[N|C^
	 */
	public void seek(long pos) throws IOException {
		seekp = pos;
	}

	/**
	 * oCgf[^ǂݍށD
	 * őŔzŎw肵TCYoCgǂݍŊi[D
	 * ǂݍ񂾌AV[N|C^ړD
	 * @param b obt@
	 * @return ۂɓǂݍ܂ꂽf[^(P:oCg)
	 * @throws IOException ǂݍ݂Ɏsꍇ̗O
	 */
	public int read(byte[] b) throws IOException {
		try {
			DomainDataStream ds = proxy.read(seekp, b.length);

			// hCt@CVXẻ񓚂߂lƂĐݒ肷
			System.arraycopy(ds.getStream(), 0, b, 0, ds.getSize());

			int rb = ds.getSize();
			seekp += rb;

			return rb;
		} catch(NoSuchAreaException e) {
			// R[h͈͊ȌꍇAEOFExceptionɕϊ
			throw new EOFException("R[h͈͊O");
		}
	}

	/**
	 * t@CŜǂݍށD
	 * Ȃƕs\Ȃ̂ŁȂ͋֎~D<br>
	 * ̃\bhĂяoƁAOȂUnsupportedOperationExceptionD
	 * @param b obt@
	 * @throws IOException ͗O
	 */
	public void readFully(byte[] b) throws IOException {
		throw new UnsupportedOperationException();
	}

	/**
	 * 4oCgǂݍ(int)D
	 * ǂݍ񂾌AV[N|C^ړD
	 * @return ǂݍ񂾃f[^
	 * @throws IOException ͗O
	 */
	public int readInt() throws IOException {
		byte[] intbuf = new byte[4];
		int readcnt = read(intbuf);

		if(readcnt < intbuf.length) {
			throw new EOFException();
		}

		return ByteStreamConverter.getInt(intbuf);
	}

	/**
	 * 8oCgǂݍ(long)D
	 * ǂݍ񂾌AV[N|C^ړD
	 * @return ǂݍ񂾃f[^
	 * @throws IOException ͗O
	 */
	public long readLong() throws IOException {
		byte[] longbuf = new byte[8];
		int readcnt = read(longbuf);

		if(readcnt < longbuf.length) {
			throw new EOFException();
		}

		return ByteStreamConverter.getLong(longbuf);
	}

	/**
	 * oCgށD
	 * 񂾌AV[N|C^ړD
	 * @param b oCg
	 * @throws IOException o͗O
	 */
	public void write(byte[] b) throws IOException {
		proxy.write(seekp, b);
		seekp += b.length;
	}

	/**
	 * 1oCgށD
	 * 񂾌AV[N|C^ړD
	 * @param b ރf[^
	 * @throws IOException o͗O
	 */
	public void write(int b) throws IOException {
		write(new byte[]{(byte)b});
	}

}
