/*
 * LOGICAL-PARADOX.ORG
 * Copyright (C)2005 satoshi akabane(akabane@logical-paradox.org)
 * $Id: DumpListEditor.java,v 1.2 2005/09/07 13:38:06 rampil Exp $
 */
package org.logical_paradox.vsan.tools.console.util;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import org.logical_paradox.common.util.StringUtils;

/**
 * _vXg쐬
 * @author satoshi akabane@logical-paradox.org
 * @version $Revision: 1.2 $
 */
public class DumpListEditor {
	/** tH[}bg = 16i */
	public static final int DUMP_HEX = 1;
	/** o͒P = 1oCg */
	public static final int UNIT_BYTE = 1;
	/** o͒P = 2oCg */
	public static final int UNIT_WORD = 2;
	/** o͒P = 4oCg */
	public static final int UNIT_LONG = 4;
	/** o͒P = 8oCg */
	public static final int UNIT_LONG_LONG = 8;

	/** oCgI[_[ = gGfBA */
	public static final int ORDER_LITTLE_ENDIAN = -1;
	/** oCgI[_[ = rbOGfBA */
	public static final int ORDER_BIG_ENDIAN = 1;

	/** IꂽtH[}bg */
	private final int format;
	/** Iꂽjbg */
	private final int unit;
	/** IꂽoCgI[_[ */
	private final int byteOrder;
	/** 1sɏo͂郆jbg */
	private final int uol;

	/**
	 * RXgN^
	 * 16i/oCgP/bigendian/1s=32unitsŕ\
	 */
	public DumpListEditor() {
		this(DUMP_HEX, UNIT_BYTE, ORDER_BIG_ENDIAN, 32);
	}
	/**
	 * RXgN^
	 * @param fmt tH[}bg
	 * @param ut jbg
	 * @param bo oCgI[_[
	 * @param u 1sɏo͂郆jbg
	 */
	public DumpListEditor(int fmt, int ut, int bo, int u) {
		format = fmt;
		unit = ut;
		byteOrder = bo;
		uol = u;
	}
	/**
	 * tH[}bgԂ
	 * @return tH[}bg
	 */
	public int getFormat() {
		return format;
	}
	/**
	 * jbg̒PʂԂ
	 * @return jbg
	 */
	public int getUnit() {
		return unit;
	}
	/**
	 * oCgI[_[Ԃ
	 * @return oCgI[_[
	 */
	public int getByteOrder() {
		return byteOrder;
	}
	/**
	 * 1sɏo͂郆jbgԂ
	 * @return jbg
	 */
	public int getUnitsOfLine() {
		return uol;
	}
	/**
	 * _v
	 * @param bytes _vΏۃoCg
	 * @return \
	 */
	public String edit(byte[] bytes) {
		int paddings = bytes.length % unit;
		int bufferlen = bytes.length + paddings;
		ByteOrder order = getByteOrder() == ORDER_BIG_ENDIAN ? ByteOrder.BIG_ENDIAN : ByteOrder.LITTLE_ENDIAN;
		ByteBuffer buffer = ByteBuffer.allocate(bufferlen).order(order);
		buffer.put(bytes);
		// Ȃ͕₤
		if(paddings > 0) {
			buffer.put(new byte[paddings]);
		}
		buffer.rewind();

		String dd = null;

		switch(format) {
			case DUMP_HEX:						// 16i
			default:
				dd = dumpHex(buffer, bufferlen);
				break;
		}

		return dd;
	}
	/**
	 * _v
	 * @param buffer _vΏۂ̃f[^
	 * @param bufferlen obt@̃TCY(pfBOꂽ)
	 * @return _v
	 */
	protected String dumpHex(ByteBuffer buffer, int bufferlen) {
		StringBuffer sb = new StringBuffer();
		// jbgPʂŃ[v
		int loopcnt = bufferlen / unit;

		for(int i = 0; i < loopcnt; i++) {
			if(i % uol > 0) {
				sb.append(" ");
			}
			switch(unit) {
				case UNIT_BYTE:				// oCg(8bit)
					sb.append(formatColumns(Integer.toHexString(buffer.get()), 2));
					break;
				case UNIT_WORD:				// [h(16bit)
					sb.append(formatColumns(Integer.toHexString(buffer.getInt()), 4));
					break;
				case UNIT_LONG:				// O[h(32bit)
					sb.append(formatColumns(Integer.toHexString(buffer.getInt()), 8));
					break;
				case UNIT_LONG_LONG:			// OO[h(64bit)
					sb.append(formatColumns(Long.toHexString(buffer.getLong()), 16));
					break;
			}
			if((i+1) % uol == 0) {
				// s
				sb.append("\n");
			}
		}
		return sb.toString();
	}
	/**
	 * 낦
	 * ȂɂẮA'0't^
	 * @param value o͂l
	 * @param clmns 
	 * @return
	 */
	protected String formatColumns(String value, int clmns) {
		// w肳ꂽl`OɊɏ\ꍇAȂł̂܂ܕԂ
		if(value.length() >= clmns) {
			return value.substring(value.length() -2);
		}
		String str = StringUtils.sequence('0', clmns) + value;
		return str.substring(str.length() - clmns);
	}
}
